#############################################################################
#
# Apache::Session::DBIStore::Sybase
# Apache persistent user sessions in a DBI::Sybase database
#
# Copyright(c) 1999 Mark Landry (mdlandry@lincoln.midcoast.com)
# Derived by from work done by Jeffrey William Baker (jeffrey@kathyandjeffrey.net)
# Distribute under the Artistic License
#
############################################################################

package Apache::Session::DBIStore::Sybase;

use strict;
use vars qw(@ISA);

@ISA = qw(Apache::Session::DBIStore);

use Apache::Session::DBIStore;
use Storable qw(freeze thaw);

sub connection {
    my $self    = shift;
    my $session = shift;
    
    return if (defined $self->{dbh});

    $self->SUPER::connection($session);

    $self->{dbh}->do(qq{set textsize 0}); # set default to 32KB
}

sub insert {
    my $self    = shift;
    my $session = shift;
 
    $self->connection($session);

    my $serialized = '0x' . unpack('H*', freeze $session->{data} );
    my $len_serialized = (length($serialized) / 2) - 1;
    
    my $sth = $self->{dbh}->prepare(qq{
                INSERT INTO sessions (id, length, a_session)
                VALUES
('$session->{data}->{_session_id}',$len_serialized,$serialized)});

    $sth->execute;

    $sth->finish;
}

sub update {
    my $self    = shift;
    my $session = shift;
 
    $self->connection($session);

    my $serialized = '0x' . unpack('H*', freeze($session->{data}));
    my $len_serialized = (length($serialized) / 2) - 1;

    my $sth = $self->{dbh}->prepare(qq{
                UPDATE sessions SET length = $len_serialized, a_session
= $serialized
		    WHERE id = '$session->{data}->{_session_id}'});
    
    $sth->execute;

    $sth->finish;
}

sub materialize {
    my $self    = shift;
    my $session = shift;

    $self->connection($session);

    if (!defined $self->{materialize_sth}) {
        $self->{materialize_sth} = 
            $self->{dbh}->prepare_cached(qq{
                SELECT a_session FROM sessions WHERE id = ?});
    }
    
    $self->{materialize_sth}->bind_param(1,
$session->{data}->{_session_id});
    
    $self->{materialize_sth}->execute;
    
    my $results = $self->{materialize_sth}->fetchrow_arrayref;

    if (!(defined $results)) {
        die "Object does not exist in the data store";
    }

    $self->{materialize_sth}->finish;

    $session->{data} = thaw(pack('H*', $results->[0]));
}

1;
